﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using Windows.Data.Xml.Dom;
using Windows.Web.Syndication;
using System.Text.RegularExpressions;
using System.IO;

namespace ASPItalia.Apps4Windows.WinRT.Services
{
	/// <summary>
	/// ENG: This class loads the items.
	/// ITA: Gestisce il caricamento dei contenuti.
	/// </summary>
	public abstract class FeedLoader : ILoader
	{
		public abstract Task<IEnumerable<FeedItem>> LoadItemsAsync();

		protected IEnumerable<FeedItem> LoadFromXML(XmlDocument feedXML)
		{
			// ENG: get the items from the XML feed
			// ITA: Leggo gli elementi dal feed XML
			SyndicationFeed feed = new SyndicationFeed();
			feed.LoadFromXml(feedXML);

			// ENG: let's map it with LINQ
			// ITA: mapping con LINQ
			return feed.Items.ToList().Select(x =>
				new FeedItem
				{
					Title = x.Title.ExtractText(),
					Uri = x.GetUri(),
					Id = (x.ItemUri != null) ? x.ItemUri.ToString() : x.Title.ExtractText(),
					PubDate = x.PublishedDate.ToLocalTime().DateTime,
					Content = x.GetContent(),
					PlainTextContent = x.GetContent().ToPlainText(),
					Subtitle = string.Empty,
					ContentType = x.GetContentType(),
					Image = GetImageUri(x),
					Video = GetVideoUri(x),
				}).ToList();
		}

		private Uri GetImageUri(SyndicationItem x)
		{
			var uri = x.Links.Where(y => y.MediaType.StartsWith("image"))
												.Select(y => y.Uri).FirstOrDefault();

			// ENG: try to get the image from the HTML
			// ITA: proviamo a recuperare l'immagine dall'HTML
			if (uri == null)
			{
				string s = GetImageFromPostContents(x);
				if (!String.IsNullOrEmpty(s))
					uri = new Uri(s);
			}
			// ENG: fix for local urls
			// ITA: fix per url locali
			if (uri != null && uri.IsLocalOrEmpty())
			{
				return new Uri("ms-appx://" + uri.AbsolutePath);
			}

			if (uri == null || !DisplayImages)
			{
				return new Uri("ms-appx:///Assets/Icons/Dark/" + this.IconType.ToString() + ".png");
			}

			return uri;
		}

		private Uri GetVideoUri(SyndicationItem x)
		{
			var uri = x.Links.Where(y => y.MediaType.StartsWith("video"))
												.Select(y => y.Uri).FirstOrDefault();

			return uri;
		}

		private static string GetImageFromPostContents(SyndicationItem item)
		{
			if (item.Summary == null)
				return null;

			// ENG: let's try with links pointing to images
			// then we'll try with images directly
			// ITA: Cerco prima i link che puntano ad immagini
			// Poi tento con le immagini direttamente
			string[] patterns = new[] {
                "href\\s*=\\s*(?:\"(?<1>[^\"]*)\"|(?<1>\\S+))",
                "src\\s*=\\s*(?:\"(?<1>[^\"]*)\"|(?<1>\\S+))"
            };

			foreach (string p in patterns)
			{
				var s = Regex.Matches((item.Summary ?? item.Content).Text,
						p,
						RegexOptions.None)
					.Cast<Match>()
					.Where(m =>
					{
						Uri url;
						if (Uri.TryCreate(m.Groups[1].Value, UriKind.Absolute, out url))
						{
							string ext = Path.GetExtension(url.AbsolutePath).ToLower();
							if (ext == ".png" || ext == ".jpg") return true;
						}
						return false;
					})
					.Select(m => m.Groups[1].Value)
					.FirstOrDefault();
				if (s != null)
					return s;
			}

			return null;
		}

		public async Task<List<FeedItem>> Search(string searchKey)
		{
			var result = await this.LoadItemsAsync();

			return result.Where(x => x.Content.ToLower().Contains(searchKey) ||
				x.Title.ToLower().Contains(searchKey)).ToList();
		}

		public IconTypeEnum IconType { get; set; }
		public bool DisplayImages { get; set; }
	}
}
