﻿using System;
using System.Linq;
using System.IO;
using System.Runtime.Serialization;
using System.Text.RegularExpressions;
using ASPItalia.Apps4Windows.WinRT.Model;
using Windows.Web.Syndication;
using ASPItalia.Apps4Windows.WinRT.Services;
using System.Net;
using Windows.UI.Xaml.Controls;
using ASPItalia.Apps4Windows.WinRT.Pages;
using Windows.Storage.Streams;
using Windows.ApplicationModel.DataTransfer;
using System.Collections.Generic;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using ASPItalia.Apps4Windows.WinRT.Resources;

namespace ASPItalia.Apps4Windows.WinRT.Extensions
{
    public static class Utils
    {
        /// <summary>
		/// ENG: get the text from ISyndicationText
        /// ITA: Estrae il testo da un ISyndicationText
        /// </summary>
        /// <param name="syndicationText"></param>
        /// <returns></returns>
        public static string ExtractText(this ISyndicationText syndicationText)
        {
            return syndicationText != null ? WebUtility.HtmlDecode(syndicationText.Text) : string.Empty;
        }

        private static Regex _cleanerRegex = new Regex(@"</?\w+((\s+\w+(\s*=\s*(?:"".*?""|'.*?'|[^'"">\s]+))?)+\s*|\s*)/?>");
        /// <summary>
		/// ENG: removes HTML and get a plain text version
        /// ITA: Rimuove l'HTML ed ottengo una versione plain text
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public static string ToPlainText(this string html)
        {
            return _cleanerRegex.Replace(html, String.Empty).Trim();
        }

        /// <summary>
		/// ENG: returns true if the text contains HTML
        /// ITA: controllo se un testo contiene HTML
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public static bool GetIsHtml(this string html)
        {
            Regex r = new Regex(@"\<\w+\>.*?\</\w+\>");
            return r.IsMatch(html);
        }


        public static string SerializeValues(params object[] values)
        {
            DataContractSerializer serializer = new DataContractSerializer(typeof(object[]), GetKnownTypes());

            using (MemoryStream stream = new MemoryStream())
            {
                serializer.WriteObject(stream, values);
                stream.Position = 0;

                return new StreamReader(stream).ReadToEnd();
            }
        }

        public static object[] DeserializeValues(string valuesString)
        {
            DataContractSerializer serializer = new DataContractSerializer(typeof(object[]), GetKnownTypes());
            using (MemoryStream stream = new MemoryStream())
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(valuesString);
                writer.Flush();
                stream.Position = 0;

                return (object[])serializer.ReadObject(stream);
            }
        }

        private static Type[] GetKnownTypes()
        {
            return new[] { typeof(SectionNames), typeof(FeedItem) };
        }

        /// <summary>
		/// ENG: get an URI from ISyndicationText
		/// ITA: estrae un URI da un ISyndicationText
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public static Uri GetUri(this SyndicationItem item)
        {
            return item.ItemUri ?? item.Links
                        .Where(x => string.IsNullOrWhiteSpace(x.MediaType))
                        .Select(y => y.Uri)
                        .FirstOrDefault();
        }

        /// <summary>
		/// ENG: get the content type from ISyndicationText
		/// ITA: estrae il content type da un ISyndicationText
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public static ContentType GetContentType(this SyndicationItem item)
        {
			// ENG: special case for maps
            // ITA: special case per le mappe
            if (item.Categories.Any(x => x.NodeValue == "map"))
                return ContentType.Map;

            if (item.Content.ExtractText().GetIsHtml() || item.GetUri() != null)
                return ContentType.Html;

            return ContentType.PlainText;
        }

        /// <summary>
        /// ENG: get the content
		/// ITA: estrae il contenuto
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public static string GetContent(this SyndicationItem item)
        {
            var result = item.Content.ExtractText();
            if (string.IsNullOrWhiteSpace(result))
            {
                result = item.Summary.ExtractText();
            }

            return result;
        }

        /// <summary>
		/// ENG: returns true if the URI is empty or local
        /// ITA: indica se l'URI è vuoto o locale
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        public static bool IsLocalOrEmpty(this Uri uri)
        {
            return uri == null || uri.ToString().IndexOf("http://local/", StringComparison.CurrentCultureIgnoreCase) > -1 ||
                uri.ToString().StartsWith("ms-appx://");
        }

        public static string ToFileName(this string name)
        {
            return name.GetHashCode().ToString();
        }

        public static string ToFileName(this Uri source)
        {
            var result = new string(source.ToString()
                .ToCharArray()
                .Where(x => char.IsLetterOrDigit(x))
                .ToArray());

            return Path.ChangeExtension(result, ".cache");
        }

		/// <summary>
		/// ENG: navigate to the section
		/// ITA: naviga alla sezione
		/// </summary>
		/// <param name="frame"></param>
		/// <param name="section"></param>
        public static void NavigateToSection(this Frame frame, Section section)
        {
            string parameter = Utils.SerializeValues(section.Name);

			// ENG: navigates to the section page, given the template
            // ITA: navigazione alla pagina di sezione, in base al tipo di template scelto dall'utente
            switch (section.Type)
            {
                case SectionTypeEnum.List:
                    frame.Navigate(typeof(ListPage), parameter);
                    break;
                case SectionTypeEnum.VideoGallery:
                case SectionTypeEnum.PhotoGallery:
                case SectionTypeEnum.Grid:
                    frame.Navigate(typeof(GridPage), parameter);
                    break;
                case SectionTypeEnum.News:
                    frame.Navigate(typeof(NewsGridPage), parameter);
                    break;
            }
        }

		/// <summary>
		/// ENG: navigates to the content
		/// ITA: naviga al dettaglio
		/// </summary>
		/// <param name="frame"></param>
		/// <param name="item"></param>
        public static void NavigateToDetails(this Frame frame, SectionItem item)
        {
            string parameter = Utils.SerializeValues(item.Section.Name, item.InnerItem.Id);
            frame.Navigate(typeof(ItemDetailPage), parameter);
        }


        /// <summary>
        /// ENG: set the share contract
		/// ITA: imposta lo share contract
        /// </summary>
        /// <param name="e"></param>
        /// <param name="item"></param>
        internal static void ShareDetail(DataRequestedEventArgs e, SectionItem item)
        {
            if (item == null)
            {
				// ENG: no elements, let's notify the user
                // ITA: non ci sono elementi selezionati, avviso l'utente
                e.Request.FailWithDisplayText(StringDictionary.NothingToShare);
                return;
            }

			// ENG: let's set the properties
            // ITA: impostiamo le proprietà di selezione
            e.Request.Data.Properties.Title = item.Title;
            e.Request.Data.Properties.Description = item.Description;
            if (item.Image != null)
            {
                var bitmap = RandomAccessStreamReference.CreateFromUri(item.Image);
                e.Request.Data.Properties.Thumbnail = bitmap;
                e.Request.Data.SetBitmap(bitmap);
            }

			// ENG: build the text
            // ITA: costruisco il testo di share
            var shareText = e.Request.Data.Properties.Title;
            var shareHtml = "<h1>" + e.Request.Data.Properties.Title + "</h1>";

			// ENG: set the URI, if available
            // ITA: condivido l'URI, se disponibile
            if (!item.Uri.IsLocalOrEmpty())
            {
                e.Request.Data.SetUri(item.Uri);
                shareText += " " + item.Uri;
                shareHtml += "<a href=\"" + item.Uri + "\">" + StringDictionary.Details + "</a>";
            }
            else if (item.Image != null && !item.Image.IsLocalOrEmpty())
            {
				// ENG: set the image
                // ITA: condivido l'immagine
                shareText += " " + item.Image.ToString();
                shareHtml += "<img src=\"" + item.Image.ToString() + "\" />";
            }

			// ENG: share the HTML, if available
            // ITA: convido l'HTML, se disponibile
            if (!string.IsNullOrEmpty(item.Content))
                e.Request.Data.SetHtmlFormat(HtmlFormatHelper.CreateHtmlFormat(shareHtml + item.Content));
            else
				// ENG: share the text
                // ITA: convido il testo
                e.Request.Data.SetText(shareText);
        }

        /// <summary>
		/// ENG: parse and get the URI's querystring
        /// ITA: Effettua il parsing della querystring di un URI
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        public static IDictionary<String, String> ParseQueryString(this Uri uri)
        {
            string[] q = uri.Query.Split(new[] { '?', '&' });
            return q.Select(p => p.Split('='))
                .Where(p => p.Length == 2)
                .ToDictionary(p => WebUtility.UrlDecode(p[0]), p => WebUtility.UrlDecode(p[1]));
        }

        public static IEnumerable<DependencyObject> GetDescendants(this DependencyObject element)
        {
            if (element == null)
                throw new ArgumentNullException("element");

            for (int x = 0; x < VisualTreeHelper.GetChildrenCount(element); x++)
            {
                DependencyObject child = VisualTreeHelper.GetChild(element, x);
                yield return child;

                foreach (DependencyObject d in GetDescendants(child))
                    yield return d;
            }
        }
    }
}